From b6ad8e07eaa21ee504ae84c49a4f3f3ba29dd309 Mon Sep 17 00:00:00 2001
From: Roger Sayle <roger@nextmovesoftware.com>
Date: Tue, 24 Aug 2021 17:24:19 +0100
Subject: [PATCH] nvptx: Emulate clock and other machine stubs.

2021-08-24  Roger Sayle  <roger@nextmovesoftware.com>
	    Corinna Vinschen  <corinna@vinschen.de>

newlib/
	configure.host: Add -DCLOCK_PROVIDED to newlib_cflags on nvptx*.

newlib/libc/machine/nvptx
	Makefile.am: Add clock.c to lib_a_SOURCES.
	clock.c: New source file to implement/approximate clock().
	misc.c: Add stubs for fstat, isatty, open, sync and unlink.
---
 newlib/configure.host                 |  2 +-
 newlib/libc/machine/nvptx/Makefile.am |  2 +-
 newlib/libc/machine/nvptx/clock.c     | 19 +++++++++++++++++++
 newlib/libc/machine/nvptx/misc.c      | 26 ++++++++++++++++++++++++++
 4 files changed, 47 insertions(+), 2 deletions(-)
 create mode 100644 newlib/libc/machine/nvptx/clock.c

diff --git a/newlib/configure.host b/newlib/configure.host
index 4ac5334..ef481ff 100644
--- a/newlib/configure.host
+++ b/newlib/configure.host
@@ -289,7 +289,7 @@ case "${host_cpu}" in
 	;;
   nvptx*)
 	machine_dir=nvptx
-	newlib_cflags="${newlib_cflags} -DMALLOC_PROVIDED"
+	newlib_cflags="${newlib_cflags} -DCLOCK_PROVIDED -DMALLOC_PROVIDED"
 	;;
   or1k*|or1knd*)
 	machine_dir=or1k
diff --git a/newlib/libc/machine/nvptx/Makefile.am b/newlib/libc/machine/nvptx/Makefile.am
index 66664d3..b0d4c78 100644
--- a/newlib/libc/machine/nvptx/Makefile.am
+++ b/newlib/libc/machine/nvptx/Makefile.am
@@ -10,7 +10,7 @@ noinst_LIBRARIES = lib.a
 
 lib_a_SOURCES = calloc.c callocr.c malloc.c mallocr.c realloc.c reallocr.c \
 		free.c write.c assert.c puts.c putchar.c printf.c abort.c \
-		exit.c misc.c
+		exit.c misc.c clock.c
 lib_a_CFLAGS = $(AM_CFLAGS)
 
 ACLOCAL_AMFLAGS = -I ../../.. -I ../../../..
diff --git a/newlib/libc/machine/nvptx/clock.c b/newlib/libc/machine/nvptx/clock.c
new file mode 100644
index 0000000..7ca0f3a
--- /dev/null
+++ b/newlib/libc/machine/nvptx/clock.c
@@ -0,0 +1,19 @@
+/* clock.c
+ * Support file for nvptx in newlib.
+ */
+#include <time.h>
+
+clock_t
+clock ()
+{
+  unsigned long long now;
+#if __PTX_SM__ >= 310
+  asm volatile("mov.u64 %0, %%globaltimer;" : "=r"(now));
+  return now/((1000000000ull)/CLOCKS_PER_SEC);
+#else
+  asm volatile("mov.u64 %0, %%clock64;" : "=r"(now));
+  // Assume a GPU base clock frequency of 1250MHz.
+  return now/((1250000000ull)/CLOCKS_PER_SEC);
+#endif
+}
+
diff --git a/newlib/libc/machine/nvptx/misc.c b/newlib/libc/machine/nvptx/misc.c
index ed1bc34..3609241 100644
--- a/newlib/libc/machine/nvptx/misc.c
+++ b/newlib/libc/machine/nvptx/misc.c
@@ -15,6 +15,7 @@
 
 #include <errno.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #undef errno
 extern int errno;
 
@@ -23,12 +24,37 @@ close(int fd) {
   return -1;
 }
 
+int
+fstat (int fd, struct stat *buf) {
+  return -1;
+}
+
+int
+isatty (int fd) {
+  return fd == 1;
+}
+
 off_t
 lseek(int fd, off_t offset, int whence) {
   return 0;
 }
 
 int
+open (const char *pathname, int flags, ...) {
+  return -1;
+}
+
+int
 read(int fd, void *buf, size_t count) {
   return 0;
 }
+
+void
+sync (void) {
+}
+
+int
+unlink (const char *pathname) {
+  return -1;
+}
+
-- 
1.8.3.1

